@@ -406,9 +406,13 @@ gxps_archive_read_entry (GXPSArchive *archive,
 	gboolean      retval;
 
 	stream = gxps_archive_open (archive, path);
-	if (!stream)
-		/* TODO: Error */
+	if (!stream) {
+                g_set_error (error,
+                             G_IO_ERROR,
+                             G_IO_ERROR_NOT_FOUND,
+                             "The entry '%s' was not found in archive", path);
 		return FALSE;
+        }
 
 	entry_size = archive_entry_size (GXPS_ARCHIVE_INPUT_STREAM (stream)->entry);
 	if (entry_size <= 0) {
@@ -423,7 +427,7 @@ gxps_archive_read_entry (GXPSArchive *archive,
 		*buffer = g_malloc (buffer_size);
 		do {
 			bytes = g_input_stream_read (stream, &buf, BUFFER_SIZE, NULL, error);
-			if (*error != NULL) {
+			if (bytes < 0) {
 				g_free (*buffer);
 				g_object_unref (stream);
 
@@ -441,7 +445,10 @@ gxps_archive_read_entry (GXPSArchive *archive,
 		g_object_unref (stream);
 
 		if (*bytes_read == 0) {
-			/* TODO: Error */
+                        g_set_error (error,
+                                     G_IO_ERROR,
+                                     G_IO_ERROR_INVALID_DATA,
+                                     "The entry '%s' is empty in archive", path);
 			g_free (*buffer);
 			return FALSE;
 		}
@@ -513,6 +520,13 @@ gxps_archive_input_stream_read (GInputStream  *stream,
 		return -1;
 
         bytes_read = archive_read_data (istream->zip->archive, buffer, count);
+        if (bytes_read < 0) {
+                g_set_error_literal (error,
+                                     G_IO_ERROR,
+                                     g_io_error_from_errno (archive_errno (istream->zip->archive)),
+                                     archive_error_string (istream->zip->archive));
+                return -1;
+        }
         if (bytes_read == 0 && istream->is_interleaved && !gxps_archive_input_stream_is_last_piece (istream)) {
                 /* Read next piece */
                 gxps_archive_input_stream_next_piece (istream);
