@@ -106,6 +106,8 @@ glyphs_indices_token_type_to_string (GlyphsIndicesTokenType type)
         default:
                 g_assert_not_reached ();
         }
+
+        return NULL;
 }
 
 static gboolean
@@ -195,7 +197,6 @@ glyphs_lookup_index (cairo_scaled_font_t *scaled_font,
         cairo_glyph_t stack_glyphs[1];
         cairo_glyph_t *glyphs = stack_glyphs;
         int num_glyphs = 1;
-        int utf8_len = g_utf8_next_char (utf8) - utf8;
         gulong index = 0;
 
         if (utf8 == NULL || *utf8 == '\0')
@@ -203,7 +204,8 @@ glyphs_lookup_index (cairo_scaled_font_t *scaled_font,
 
         status = cairo_scaled_font_text_to_glyphs (scaled_font,
                                                    0, 0,
-                                                   utf8, utf8_len,
+                                                   utf8,
+                                                   g_utf8_next_char (utf8) - utf8,
                                                    &glyphs, &num_glyphs,
                                                    NULL, NULL, NULL);
 
@@ -362,6 +364,7 @@ glyphs_indices_parse (const char          *indices,
                         break;
                 case GI_TOKEN_EOF:
                         eof = TRUE;
+                        /* fall through */
                 case GI_TOKEN_SEMICOLON: {
                         cairo_text_extents_t extents;
 
